// Copyright 2011 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "bookmarkmanager.h"
#include "ui_bookmarkmanager.h"

#include <QWebSettings>
#include <QMessageBox>

BookmarkManager::BookmarkManager(BookmarkFolder* root, QWidget *parent) : QDialog(parent), ui(new Ui::BookmarkManager) {
	ui->setupUi(this);

	m_manageMode = true;
	m_rootFolder = root;
	m_stack.append(m_rootFolder);

	ui->bBack->setEnabled(false);

	//Create Add folder widget
	m_frAddFolder = new QFrame();
	m_tfFolderName = new QLineEdit();
	m_bAddFolder = new QPushButton(tr("Add folder"));

	m_frAddFolder->setLayout(new QHBoxLayout());
	m_frAddFolder->layout()->addWidget(m_tfFolderName);
	m_frAddFolder->layout()->addWidget(m_bAddFolder);

	connect(ui->bClose, SIGNAL(clicked()), this, SLOT(reject()));
	connect(ui->bBack, SIGNAL(clicked()), this, SLOT(goBack()));

	connect(m_bAddFolder, SIGNAL(clicked()), this, SLOT(createFolder()));

	showContent();
#if defined(Q_WS_WIN)
	ui->bBack->setFlat(false);
#endif
}

BookmarkManager::~BookmarkManager() {
	delete ui;
}

void BookmarkManager::addBookmark(QString title, QString url) {
	bmUrl = url;
	bmTitle = title;
	m_manageMode = false;

	m_stack.clear();
	m_stack.append(m_rootFolder);

	showContent();

	exec();
}

void BookmarkManager::manageBookmarks() {
	m_stack.clear();
	m_stack.append(m_rootFolder);
	m_manageMode = true;

	showContent();

	exec();
}

void BookmarkManager::accept() {
	Bookmark* b = new Bookmark();
	b->setTitle(bmTitle);
	b->setUrl(QUrl(bmUrl));
	b->setIcon(QWebSettings::iconForUrl(b->url()));

	m_stack.last()->addBookmark(b);

	QDialog::accept();
}

void BookmarkManager::goInto(BookmarkFolder *f) {
	m_stack.append(f);
	showContent();
	ui->bBack->setEnabled(true);
}

void BookmarkManager::deleteBookmark(BookmarkButton *bmb) {
	m_stack.last()->removeBookmark(bmb->bookmark());
	m_currentContents.removeAll(bmb);
	delete bmb->bookmark();
	ui->items->removeWidget(bmb);
	bmb->deleteLater();
}

void BookmarkManager::deleteBookmarkFolder(BookmarkFolderButton *bmfb) {
	m_stack.last()->removeFolder(bmfb->folder());
	m_currentContents.removeAll(bmfb);
	delete bmfb->folder();
	ui->items->removeWidget(bmfb);
	bmfb->deleteLater();
}

void BookmarkManager::goBack() {
	if (m_stack.size() < 2) return;
	m_stack.removeLast();
	showContent();

	if (m_stack.size() < 2) ui->bBack->setEnabled(false);
}

void BookmarkManager::showContent() {
	//1. remove old content
	while (!m_currentContents.isEmpty()) {
		QWidget* w = m_currentContents.value(0);
		ui->items->removeWidget(w);
		m_currentContents.removeAll(w);
		if (w != m_frAddFolder) delete w;
	}

	//Hide the real-name for the root folder
	if (m_stack.size() == 1) ui->lFolderName->setText(tr("Bookmarks"));
	else ui->lFolderName->setText(m_stack.last()->name());

	//2. create new contents
	BookmarkFolder* f = m_stack.last();
	QList<BookmarkFolder*> subfolders = f->subfolders();
	QList<Bookmark*> bookmarks = f->bookmarks();

	//First the folders
	for (int i = 0; i < subfolders.size(); i++) {
		BookmarkFolderButton* bfb = new BookmarkFolderButton(subfolders.value(i), m_manageMode);
		ui->items->addWidget(bfb);
		m_currentContents.append(bfb);
		connect(bfb, SIGNAL(jumpedInto(BookmarkFolder*)), this, SLOT(goInto(BookmarkFolder*)));
		connect(bfb, SIGNAL(toBeDeleted(BookmarkFolderButton*)), this, SLOT(deleteBookmarkFolder(BookmarkFolderButton*)));
	}

	//Show option to add a folder
	ui->items->addWidget(m_frAddFolder);
	m_currentContents.append(m_frAddFolder);

	if (!m_manageMode) {
		//Add a "Insert here" button here
		QPushButton* b = new QPushButton(tr("Add bookmark here"));
		b->setStyleSheet("text-align:left");
		ui->items->addWidget(b);
		m_currentContents.append(b);
		connect(b, SIGNAL(clicked()), this, SLOT(accept()));
	}

	//At last the bookmarks
	for (int i = 0; i < bookmarks.size(); i++) {
		BookmarkButton* b = new BookmarkButton(bookmarks.value(i), m_manageMode);
		connect(b, SIGNAL(toBeDeleted(BookmarkButton*)), this, SLOT(deleteBookmark(BookmarkButton*)));

		ui->items->addWidget(b);
		m_currentContents.append(b);
	}
}

void BookmarkManager::createFolder() {
	QString name = m_tfFolderName->text();
	if (name == "") return;

	//Check if the name is already given
	QList<BookmarkFolder*> folders = m_stack.last()->subfolders();
	for (int i = 0; i < folders.size(); i++) {
		if (folders.value(i)->name() == name) {
			QMessageBox::information(this, tr("Create folder"), tr("There is already a folder with that name"));
			return;
		}
	}

	BookmarkFolder* bmf = new BookmarkFolder();
	bmf->setName(name);
	bmf->addAllAction();

	m_stack.last()->addFolder(bmf, true);

	m_tfFolderName->clear();

	showContent();
}
