// Copyright 2011 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "lightweight.h"
#include "ui_lightweight.h"

#include <QFile>
#include <QDir>
#include <QShortcut>
#include <QCryptographicHash>
#include <QSettings>

#include <QDebug>

Lightweight::Lightweight(QWidget *parent) : QWidget(parent), ui(new Ui::Lightweight) {
	ui->setupUi(this);

	tabs = new QStackedLayout();
	tabList = new QList<BrowserTab*>();
	ui->verticalLayout->addLayout(tabs);

	ui->verticalLayout->setSpacing(0);
	currentTab = 0;
	fullscreen = false;

	//Create downloadmanager
	downloads = new DownloadList(this);
	tabs->addWidget(downloads);
	ui->bDownloads->hide();

	//Load bookmarks from xml file
	QFile f(QDir::homePath()+"/.lightweight2/bookmarks.xml");
	QDomDocument d;
	d.setContent(&f);
	bookmarks = BookmarkFolder::readFolder(this, d.documentElement(), true);

	//Load bookmarks from old fileformat into the root folder
	QFile fi(QDir::homePath()+"/.lightweight2/bookmarks");
	if (fi.exists()) {
		fi.open(QFile::ReadOnly);
		QString line = QString::fromUtf8(fi.readLine().data()).replace("\n", "");
		while (line != "END") {
			QString title = line;
			QString url = QString::fromUtf8(fi.readLine().data()).replace("\n", "");

			Bookmark* b = new Bookmark();
			b->setTitle(title);
			b->setUrl(QUrl(url));
			b->setIcon(Bookmark::iconFor(url));
			bookmarks->addBookmark(b);

			line = QString::fromUtf8(fi.readLine().data()).replace("\n", "");
		}
		fi.close();

		//Save the bookmarks and then delete the old file
		saveBookmarks();
		fi.remove();
	}

	bmm = new BookmarkManager(bookmarks,0);

	ui->bNewTab->setMenu(bookmarks->action()->menu());
	bookmarks->emptyAction()->setShortcut(QKeySequence(tr("Ctrl+T")));
	ui->bNewTab->setDefaultAction(bookmarks->emptyAction());

	connect(bookmarks->action()->menu(), SIGNAL(triggered(QAction*)), this, SLOT(openBookmark(QAction*)));
	connect(bookmarks->action()->menu(), SIGNAL(triggered(QAction*)), bookmarks->action()->menu(), SLOT(hide()));

	connect(ui->bBack, SIGNAL(clicked()), this, SLOT(back()));
	connect(ui->bForward, SIGNAL(clicked()), this, SLOT(forward()));
	connect(ui->bStop, SIGNAL(clicked()), this, SLOT(stop()));
	connect(ui->bReload, SIGNAL(clicked()), this, SLOT(reload()));

	connect(ui->tfSearch, SIGNAL(returnPressed()), this, SLOT(search()));

	connect(ui->bDownloads, SIGNAL(clicked()), this, SLOT(showDownloads()));
	connect(downloads, SIGNAL(hasDownloads(bool)), ui->bDownloads, SLOT(setVisible(bool)));
	connect(downloads, SIGNAL(downloadRunning(bool)), this, SLOT(setDownloadIcon(bool)));

	//Create shortcuts
	new QShortcut(QKeySequence(tr("F11")), this, SLOT(switchFullscreen()));
	new QShortcut(QKeySequence(tr("Ctrl+Tab")), this, SLOT(nextTab()));
	new QShortcut(QKeySequence(tr("Ctrl+F4")), this, SLOT(closeCurrentTab()));
	new QShortcut(QKeySequence(tr("Ctrl+F")), this, SLOT(showSearchBar()));
	new QShortcut(QKeySequence(tr("Ctrl+B")), this, SLOT(openBMM()));

	//Default search engine
	ui->cbSearchEngine->addItem(QIcon(":/icons/search/google.png"), QString(), QVariant(tr("http://www.google.com/search?q=%{query}&ie=UTF-8&oe=UTF-8")));
	ui->cbSearchEngine->addItem(QIcon(":/icons/search/wikipedia.png"), QString(), QVariant(tr("http://en.wikipedia.org/wiki/%{query}")));

	//Get user search engines
	QDir dir(QDir::homePath()+"/.lightweight2/searchengines");
	QStringList subDirs = dir.entryList(QDir::Dirs, QDir::Name);
	for (int i = 0; i < subDirs.size(); i++) {
		QString subDirStr = (QString) subDirs.value(i);
		QFile file(QDir::homePath()+"/.lightweight2/searchengines/"+subDirStr+"/info");
		if (!file.exists()) continue;
		file.open(QFile::ReadOnly);
		QString url = file.readLine().replace("\n", "");
		ui->cbSearchEngine->addItem(QIcon(QDir::homePath()+QString("/.lightweight2/searchengines/%1/icon.png").arg(subDirStr)), QString(), QVariant(url));
		file.close();
	}

	//Set the last used search engine
	QSettings s;
	ui->cbSearchEngine->setCurrentIndex(s.value("searchengine", "0").toInt());

	//restore window-pos ans size
	this->move(s.value("windowpos", QVariant(QPoint(0,0))).toPoint());
	this->resize(s.value("windowsize", QVariant(QSize(640,480))).toSize());

	//Create default tab
	newTab("about:blank", QIcon(":/icons/new.png"));

#if defined(Q_WS_WIN)
	ui->bBack->setFlat(false);
	ui->bForward->setFlat(false);
	ui->bReload->setFlat(false);
	ui->bStop->setFlat(false);
	ui->bDownloads->setFlat(false);
#endif
}

Lightweight::~Lightweight() {
	delete ui;
	while (tabList->size() > 0) {
		BrowserTab* tab = tabList->first();
		tabList->removeAll(tab);
		delete tab;
	}
	delete tabList;
}

void Lightweight::openUrl(QString url) {
	if ((currentTab != 0) && currentTab->isEmptyPage())
		currentTab->goTo(url, QIcon(), true);
	else newTab(url, QIcon(), true);
}

void Lightweight::closeEvent(QCloseEvent *e) {
	//Save bookmarks to file
	saveBookmarks();

	//Save settings
	QSettings s;
	s.setValue("searchengine", ui->cbSearchEngine->currentIndex());

	s.setValue("windowpos", this->pos());
	s.setValue("windowsize", this->size());

	for (int i = 0; i < tabList->size(); i++) {
		tabList->value(i)->stop();
	}

	//Goodbye
	QWidget::closeEvent(e);
}

BrowserTab* Lightweight::newTab(QString url, QIcon icon, bool hideUrlBar) {
	BrowserTab* tab = new BrowserTab();
	tabList->append(tab);
	ui->tabcontainer->layout()->addWidget(tab->tabButton);
	tabs->addWidget(tab->content);
	tabs->setCurrentWidget(tab->content);

	tab->goTo(url, icon, hideUrlBar);
	connect(tab, SIGNAL(selected(BrowserTab*)), this, SLOT(showTab(BrowserTab*)));
	connect(tab, SIGNAL(closed(BrowserTab*)), this, SLOT(closeTab(BrowserTab*)));
	connect(tab, SIGNAL(newTab(QString)), this, SLOT(newTab(QString)));
	connect(tab, SIGNAL(newWindow(QWebView**)), this, SLOT(newWindow(QWebView**)));
	connect(tab, SIGNAL(downloadStarted(QUrl)), downloads, SLOT(addDownload(QUrl)));
	connect(tab, SIGNAL(loadingStateChanged(bool)), this, SLOT(setButtonsForCurrentTab()));
	connect(tab, SIGNAL(bookmark(QString,QString)), bmm, SLOT(addBookmark(QString,QString)));

	showTab(tab);

	if (tab->isEmptyPage())
		tab->gainFocus();
	return tab;
}

void Lightweight::newWindow(QWebView **view) {
	BrowserTab* t = newTab("about:blank", QIcon(), true);

	(*view) = t->getView();
}

void Lightweight::back() {
	if (currentTab == 0) return;
	currentTab->back();
}

void Lightweight::forward() {
	if (currentTab == 0) return;
	currentTab->forward();
}

void Lightweight::stop() {
	if (currentTab == 0) return;
	currentTab->stop();
}

void Lightweight::reload() {
	if (currentTab == 0) return;
	currentTab->reload();
}

void Lightweight::search() {
	QString queryTemplate = ui->cbSearchEngine->itemData(ui->cbSearchEngine->currentIndex()).toString();

	queryTemplate = queryTemplate.replace("%{query}", ui->tfSearch->text());
	if ((currentTab != 0) && currentTab->isEmptyPage()) currentTab->goTo(queryTemplate, ui->cbSearchEngine->itemIcon(ui->cbSearchEngine->currentIndex()));
	else newTab(queryTemplate, ui->cbSearchEngine->itemIcon(ui->cbSearchEngine->currentIndex()), true);
}

void Lightweight::showTab(BrowserTab *tab) {
	tabs->setCurrentWidget(tab->content);
	currentTab = tab;
	setButtonsForCurrentTab();
}

void Lightweight::setButtonsForCurrentTab() {
	if (currentTab == 0) {
		ui->bBack->setEnabled(false);
		ui->bForward->setEnabled(false);
		ui->bReload->hide();
		ui->bStop->show();
		ui->bStop->setEnabled(false);
	} else {
		ui->bStop->setEnabled(true);
		ui->bBack->setEnabled(currentTab->canGoBack());
		ui->bForward->setEnabled(currentTab->canGoForward());
		ui->bStop->setVisible(currentTab->isLoading());
		ui->bReload->setVisible(!currentTab->isLoading());
	}
}

void Lightweight::closeTab(BrowserTab *tab) {
	tabs->removeWidget(tab->content);
	ui->tabcontainer->layout()->removeWidget(tab->tabButton);

	tabList->removeAll(tab);

	if (!tabList->isEmpty() && (tab == currentTab)) {
		showTab(tabList->last());
	}

	if (tabList->isEmpty()) {
		currentTab = 0;
	}

	delete tab;
}

void Lightweight::openBookmark(QAction *ac) {
	QUrl url = ac->data().toUrl();
	QString urlStr = url.toString();

	if (urlStr.startsWith("all|")) {
		//Try to open all bookmarks in given action
		QString fName = urlStr.right(urlStr.size()-4);
		if (fName == "") return;

		BookmarkFolder* bf = bookmarks->getFolder(fName);
		if (bf == 0) return;

		openAll(bf);
		return;
	}

	QIcon icon = Bookmark::iconFor(urlStr);
	if (urlStr == "about:blank") icon = QIcon(":/icons/new.png");
	bool hideUrlBar = (urlStr != "about:blank");

	if ((currentTab != 0) && currentTab->isEmptyPage())
		currentTab->goTo(urlStr, icon, hideUrlBar);
	else newTab(urlStr, icon, hideUrlBar);
}

void Lightweight::switchFullscreen() {
	if (fullscreen) {
		this->setWindowState(Qt::WindowMaximized);
		fullscreen = false;
	} else {
		this->setWindowState(Qt::WindowFullScreen);
		fullscreen = true;
	}
}

void Lightweight::nextTab() {
	if (currentTab == 0) return;
	int i = (tabList->indexOf(currentTab)+1)%tabList->size();
	showTab(tabList->value(i));
}

void Lightweight::closeCurrentTab() {
	if (currentTab != 0) closeTab(currentTab);
}

void Lightweight::showDownloads() {
	tabs->setCurrentWidget(downloads);
	currentTab = 0;
	setButtonsForCurrentTab();
}

void Lightweight::setDownloadIcon(bool running) {
	if (running)
		ui->bDownloads->setIcon(QIcon(":/icons/download_run.png"));
	else
		ui->bDownloads->setIcon(QIcon(":/icons/download.png"));
}

void Lightweight::showSearchBar() {
	if (currentTab == 0) return;
	currentTab->showSearchBar();
}

void Lightweight::openBMM() {
	bmm->manageBookmarks();
}

void Lightweight::openAll(BookmarkFolder *folder) {
	//First all bookmarks
	QList<Bookmark*> bms = folder->bookmarks();
	for (int i = 0; i < bms.size(); i++) {
		openBookmark(bms.value(i)->action());
	}

	//Now recursive all subfolders
	QList<BookmarkFolder*> bmfs = folder->subfolders();
	for (int i = 0; i < bmfs.size(); i++) {
		openAll(bmfs.value(i));
	}
}

void Lightweight::saveBookmarks() {
	QDomDocument d("Bookmarks");
	QFile file(QDir::homePath()+"/.lightweight2/bookmarks.xml");
	QTextStream stream(&file);
	file.open(QFile::WriteOnly);
	d.appendChild(bookmarks->toXml(&d));
	d.save(stream, 4);
}
